"""
Plot Morris elementary effects
"""
import openturns as ot
import numpy as np
import matplotlib
import pylab as plt
import warnings
matplotlib.rc('text', usetex=True)
matplotlib.rcParams['text.latex.preamble'] = [r"\usepackage{amsmath}"]


class PlotEE(object):

    """
    Plot elementary effects
    -----------------------

    The class perform the plot of sensitivity indices issued from Morris class.

    Parameters
    ----------
    morris: :class:`~otmorris.Morris`
        A Morris object.

    output_marginal: int
        Index of output marginal of interest.
        Default value is 0

    absolute_mean: bool
        Interest is mean of absolute elementary effects .
        Default value is True

    title: str
        Title for the graph

    """

    def __init__(self,
                 result,
                 output_marginal=0,
                 absolute_mean=True,
                 title="Elementary effects",
                 **kwargs):

        # set figure
        self._fig, self._ax = plt.subplots()
        # Check input object type
        if not (hasattr(result, 'getStandardDeviationElementaryEffects') and hasattr(result, 'getClassName')):
            raise TypeError(" `result` should be of class Morris ")
        if absolute_mean:
            mean = result.getMeanAbsoluteElementaryEffects(output_marginal)
        else:
            mean = result.getMeanElementaryEffects(output_marginal)
        sigma = result.getStandardDeviationElementaryEffects(output_marginal)
        dim = len(sigma)
        input_description = map(lambda x: "X" + str(x + 1), range(dim))
        # Plot effects
        self._ax.plot(mean, sigma, 'bo')
        # Annotate points
        dmu = (plt.np.max(mean) - plt.np.min(mean)) / len(mean)
        dsg = (plt.np.max(sigma) - plt.np.min(sigma)) / len(sigma)
        for i, txt in enumerate(input_description):
            self._ax.annotate(
                txt, (mean[i] + 0.05 * dmu, sigma[i] + 0.05 * dsg))
        self._ax.set_xlabel(r"$\boldsymbol{\mu}$", fontsize=14)
        self._ax.set_ylabel(r"$\boldsymbol{\sigma}$", fontsize=14)
        self._ax.grid(True)
        self._fig.suptitle(title, fontsize=18)

    def show(self, **kwargs):
        """
        Display the graph on screen.

        Parameters
        ----------
        kwargs:
            block: bool, optional
                If true (default), block until the graph is closed.

            These parameters are passed to matplotlib.pyplot.show()
        """
        self._fig.show(**kwargs)

    def save(self, fname, **kwargs):
        """
        Save the graph as file.

        Parameters
        ----------
        fname: bool, optional
            A string containing a path to a filename from which file format is deduced.

        kwargs:
            Refer to matplotlib.figure.Figure.savefig documentation for valid keyword arguments.
        """
        self._fig.savefig(fname, **kwargs)

    def getFigure(self):
        """
        Accessor to the underlying figure object.

        Refer to matplotlib.figure.Figure for further information.
        """
        return self._fig

    def getAxes(self):
        """
        Get the list of Axes objects.

        Refer to matplotlib.axes.Axes for further information.
        """
        return self._ax

    def close(self):
        """Close the figure."""
        plt.close(self._fig)



#!/usr/bin/env python

from __future__ import print_function
import openturns as ot
import otmorris

poutre = ot.SymbolicFunction(['L', 'b', 'h', 'E', 'F'],
                             ['F * L^3 / (48 * E * b * h^3 / 12)'])

# define the model
L = ot.ParametrizedDistribution(ot.LogNormalMuSigmaOverMu(5., 0.02))
b = ot.ParametrizedDistribution(ot.LogNormalMuSigmaOverMu(2., 0.05))
h = ot.ParametrizedDistribution(ot.LogNormalMuSigmaOverMu(0.4, 0.05))
E = ot.ParametrizedDistribution(ot.LogNormalMuSigmaOverMu(3e4, 0.12))
F = ot.ParametrizedDistribution(ot.LogNormalMuSigmaOverMu(0.1, 0.20))
list_marginals = [L, b, h, E, F]
distribution = ot.ComposedDistribution(list_marginals)
distribution.setDescription(('L', 'b', 'h', 'E', 'F'))
dim = distribution.getDimension()

level_number = 4
trajectories = 10
jump_step = int(level_number / 2)
levels = [level_number] * dim

# set the bounds of the grid experiment
bound = ot.Interval(
    [marginal.computeQuantile(0.01)[0] for marginal in list_marginals],
                    [marginal.computeQuantile(0.99)[0] for marginal in list_marginals])
experiment = otmorris.MorrisExperimentGrid(levels, bound, trajectories)
experiment.setJumpStep(ot.Indices([jump_step] * dim))

# create and compute the design of experiments
input_sample = experiment.generate()
print("Morris experiment generated from grid = ")
print(input_sample)
output_sample = poutre(input_sample)

# run the Morris analysis
morris = otmorris.Morris(input_sample, output_sample, bound)
print("E(|EE|)  = ", morris.getMeanAbsoluteElementaryEffects())
print("E(EE)  = ", morris.getMeanElementaryEffects())
print("V(|EE|)^{1/2} = ", morris.getStandardDeviationElementaryEffects())



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
import otmorris

# Use Case 1 : define a k-grid level (so delta = 1/(k-1))
r = 5
# Define experiments in [0,1]^2
print("Use Case #1 : generate trajectories from regular grid")
morris_experiment = otmorris.MorrisExperimentGrid([5, 5], r)
X = morris_experiment.generate()
print("Morris experiment generated from grid = ", X)

# Validation : check that for all trajectories, each direction is choosen once
for n in range(r):
    first_element = n * (X.getDimension() + 1)
    last_element = first_element + (X.getDimension() + 1)
    dx = X[first_element + 1:last_element] - \
        X[first_element:(last_element - 1)]
    print("#%d", n + 1)
    print("dx=", dx)

# Use Case 2 : Use of initial LHS design

ot.RandomGenerator.SetSeed(1)

# Use Case : starting from an LHS
# OT does not allow yet to manage centered lhs designs, so a design has
# been generated outside OT
lhs = ot.Sample([
                [0.875, 0.575],
                [0.775, 0.775],
                [0.625, 0.925],
                [0.375, 0.625],
                [0.275, 0.975],
                [0.075, 0.825],
                [0.225, 0.325],
                [0.425, 0.225],
                [0.925, 0.475],
                [0.325, 0.725],
                [0.025, 0.275],
                [0.525, 0.175],
                [0.125, 0.375],
                [0.475, 0.025],
                [0.825, 0.675],
                [0.175, 0.525],
                [0.975, 0.875],
                [0.675, 0.075],
                [0.725, 0.125],
                [0.575, 0.425]
                ])
print("Use Case #2 : generate trajectories from initial lhs design")
print("Initial LHS design = ", lhs)

# generate designs
morris_experiment = otmorris.MorrisExperimentLHS(lhs, r)
Y = morris_experiment.generate()
print("Morris experiment generated from LHS = ", Y)

# Validation
for n in range(r):
    first_element = n * (Y.getDimension() + 1)
    last_element = first_element + (Y.getDimension() + 1)
    dy = Y[first_element + 1:last_element] - \
        Y[first_element:(last_element - 1)]
    print("#%d", n + 1)
    print("dy=", dy)



"""
    otmorris --- An OpenTURNS module
    ================================

    Contents
    --------
      'otmorris' is a module for OpenTURNS, which enables to compute the
      elementary effects using the screening Morris method

"""

import sys
if sys.platform.startswith('win'):
    # this ensures OT dll is loaded
    import openturns

from .otmorris import *

__version__ = '0.8'



#!/usr/bin/env python

from __future__ import print_function
import openturns as ot
import otmorris

# Define model
ot.RandomGenerator.SetSeed(0)
# Number of trajectories
r = 5
# Define experiments in [0,1]^2
print("Use Case #1 : generate trajectories from regular grid")
levels = ot.Indices(2)
levels.fill(5, 0)
morris_experiment = otmorris.MorrisExperimentGrid(levels, r)
grid_bound = morris_experiment.getBounds()
sample1 = morris_experiment.generate()
print("Morris experiment generated from grid = ", sample1)

print("Use Case #2 : generate trajectories from initial lhs design")
size = 20
# Generate an LHS design
dist = ot.ComposedDistribution(2 * [ot.Uniform(0, 1)])
experiment = ot.LHSExperiment(dist, size, True, False)
lhsDesign = experiment.generate()
print("Initial LHS design = ", lhsDesign)
# Generate designs
morris_experiment_lhs = otmorris.MorrisExperimentLHS(lhsDesign, r)
lhs_bound = morris_experiment_lhs.getBounds()
sample2 = morris_experiment.generate()
print("Morris experiment generated from LHS = ", sample2)

# Define model
model = ot.SymbolicFunction(["x", "y"], ["cos(x)*y + sin(y)*x + x*y -0.1"])

# Define Morris method with two designs
morrisEE1 = otmorris.Morris(sample1, model(sample1), grid_bound)
morrisEE2 = otmorris.Morris(sample2, model(sample2), lhs_bound)
print("Using level grid, E(|EE|)  = ",
      morrisEE1.getMeanAbsoluteElementaryEffects())
print("                  V(|EE|)^{1/2} = ",
      morrisEE1.getStandardDeviationElementaryEffects())

print("Using initial LHS, E(|EE|)  = ",
      morrisEE2.getMeanAbsoluteElementaryEffects())
print("                   V(|EE|)^{1/2} = ",
      morrisEE2.getStandardDeviationElementaryEffects())
